# -*- coding: utf-8 -*-
"""
Created on Fri Sep 15 20:22:44 2023

@author: kunth
"""
from __future__ import annotations
from abc import abstractmethod, ABC
import json
import requests

from telegrapi.method import Method

# Source: https://core.telegram.org/bots/api
# https://api.telegram.org/<<your_bot_api_token>>/sendMessage?chat_id=@<<channel_name>>&text=Hello



class TEngine:
    def __init__(self, token: str):
        self.token = token
        self.base_url = f'https://api.telegram.org/bot{token}'

    def send(self, message: TData) -> dict:
        global res
        url = f'{self.base_url}/{message.method}'
        res = requests.post(url, data=message.data, files=message.files)
        res = res.content.decode('utf-8')
        res = json.loads(res)
        return res



class TData(ABC):
    bot: TEngine
    chat_id: str
    _data: dict  = None
    _files: dict = None
    _method: Method = None
    _attatch_path:str = None
    _parse_mode:str = "HTML"
    
    @property
    def method(self) -> str:
        return self._method
    
    @property
    def data(self) -> dict:
        return self._data
    
    @property
    def files(self) -> dict:
        return self._files
    
    @abstractmethod
    def send(bot: TEngine): pass



class TText(TData):
    def __init__(self,bot: TEngine, text:str, chat_id:str=None, parse_mode:str='HTML'):
        super()
        self.bot = bot
        self._method = Method.sendMessage
        self.chat_id = chat_id
        self._parse_mode = parse_mode
        self._data = {
            'chat_id': self.chat_id,
            'parse_mode': self._parse_mode,
            'text': text,
        }
      
    def send(self):
        self.bot.send(self)



class TFile(TData):
    def __init__(self,bot: TEngine, filepath: str, caption:str=None, chat_id:str=None, parse_mode:str='HTML'):
        super()
        self.bot = bot
        self._method = Method.sendDocument
        self.chat_id = chat_id
        self._attatch_path = filepath
        self._parse_mode = parse_mode
        self._data = {
                'chat_id': self.chat_id,
                'parse_mode': self._parse_mode,
                'caption': caption,
            }
      
    def send(self):
        with open(self._attatch_path, 'rb') as f:
            self._files = { 'document': f }
            return self.bot.send(self)



class TVoice(TFile):
    def __init__(self,bot: TEngine, filepath: str, caption:str=None, chat_id:str=None, parse_mode:str='HTML'):
        super().__init__(bot, filepath, caption)
        self._method = Method.sendVoice

    def send(self):
        with open(self._attatch_path, 'rb') as f:
            data = f.read()
        self._files = {
            'voice': ('Message.ogg', data),
        }
        return self.bot.send(self)




# Factory
class Chat:
    
    chat_id: str
    
    def __init__(self, chat_id: str, bot: TEngine = None, parse_mode:str = 'HTML'):
        self.chat_id = chat_id
        self.parse_mode = parse_mode
        self.bot = bot
    
    def message(self, msg: str) -> TText:
        return TText(self.bot, msg, self.chat_id, self.parse_mode)
    
    def json(self, data:dict) -> TText:
        text = json.dumps(data, ensure_ascii=False, indent=4)
        return TText(self.bot, text, self.chat_id, self.parse_mode)
    
    def file(self, filepath: str, caption: str = None) -> TFile:
        return TFile(self.bot, filepath, caption,self.chat_id)

    def voice(self, filepath: str, caption: str = None) -> TVoice:
        return TFile(self.bot, filepath, caption,self.chat_id)






# -*- coding: utf-8 -*-

import os
from dotenv import load_dotenv
import json
from telegrapi import Chat, TEngine


load_dotenv()

TELEGRAM_TOKEN = os.getenv('TELEGRAM_TOKEN')
CHAT_ID = os.getenv('CHAT_ID')

        
if __name__ == '__main__':
    
    bot = TEngine(TELEGRAM_TOKEN)
    channel = Chat(CHAT_ID, bot)
    
    # to Text message into channel
    channel.message("This is a <b>bold</b> <i>text message</i>.").send()
    
    # to send file into channel
    filepath_to_send = "../telegrapi/api.py"
    caption='test sending file.'
    result = channel.file(filepath_to_send, caption).send()
    print(result)
    
    # to send text message into channel
    # message = json.dumps(result, ensure_ascii=False, indent=4)
    result = channel.json(result).send()
    print(result)
    

    # to send text message into channel
    voice_file = "./sample-voice.ogg"
    result = channel.voice(voice_file).send()
    print(result)
    


# -*- coding: utf-8 -*-

class Method:
    sendMessage:    str = 'sendMessage'
    sendDocument:   str = 'sendDocument'
    copyMessage:    str = 'copyMessage'
    forwardMessage: str = 'forwardMessage'
    sendPhoto:      str = 'sendPhoto'
    sendAudio:      str = 'sendAudio'
    sendVideo:      str = 'sendVideo'
    sendAnimation:  str = 'sendAnimation'
    sendVoice:      str = 'sendVoice'
    sendVideoNote:  str = 'sendVideoNote'
    sendMediaGroup: str = 'sendMediaGroup'
    sendLocation:   str = 'sendLocation'
    sendVenue:      str = 'sendVenue'
    sendContact:    str = 'sendContact'
    sendPoll:       str = 'sendPoll'
    sendDice:       str = 'sendDice'
    sendChatAction: str = 'sendChatAction'
    getMe:          str = 'getMe'
    logOut:         str = 'logOut'
    close:          str = 'close'




# -*- coding: utf-8 -*-

import os
import requests
import json
from dotenv import load_dotenv


load_dotenv()

TELEGRAM_TOKEN = os.getenv('TELEGRAM_TOKEN')


def ping(channel_username) -> dict:
    url = f'https://api.telegram.org/bot{TELEGRAM_TOKEN}/sendMessage?chat_id={channel_username}&text=Hello'
    res = requests.get(url)
    res = res.content.decode('utf-8')
    return json.loads(res)

if __name__ == '__main__':
    
    channel = '@votersc'
    res = ping(channel)


# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.0.0'
DESCRIPTION = 'Telegrapi (Telegram Api) is a package for using Telegram network within Python.'
LONG_DESCRIPTION = 'Telegrapi (Telegram Api) is a package for using Telegram network within Python.'

# Setting up
setup(
    name="telegrapi",
    version=VERSION,
    author="Kunthet",
    author_email="<mail@kunthet.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    #package_dir={"": "telegrapi"},
    install_requires=[],
    license='MIT',
    license_files = ('LICENSE',),
    url='https://github.com/kunthet/telegrapi',
    keywords=['python', 'telegram', 'bot', 'api', 'light', 'simple'],
    
    classifiers=[
        # "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.9",
        # "Operating System :: Unix",
        # "Operating System :: MacOS :: MacOS X",
        # "Operating System :: Microsoft :: Windows",
        "Operating System :: OS Independent",
    ],
    
    extras_require={
        "dev": ["pytest>=7.0", "twine>=4.0.2", 'dotenv']
    },
    
    python_requires=">=3.9"
)